import java.awt.*;
import java.awt.event.*;

// classe per la gestione semplificata degli eventi di una finestra
class AscoltatoreEventiFinestra extends WindowAdapter {
  public void windowClosing(WindowEvent event) {
  System.exit(0); // uscita dal programma
 }
}

public class CalcolatriceGUI extends Frame implements ActionListener {
  static final int NUMERO_MASSIMO_CIFRE = 20;
  private Calcolatrice calcolatrice;
  private boolean decimali; // flag presenza separatore decimale
  private boolean nuovoNumero; // flag nuovo numero inserito
  private TextField display = new TextField(30);
  private Button[] pulsanti = new Button[19];
  private String etichetteTasti = "1,2,3,+,4,5,6,-,7,8,9,*,0,.,+/-,/";
  private String[] tasti = etichetteTasti.split(",");
  
  // costruttore
  public CalcolatriceGUI() {
   super("Calcolatrice"); // titolo finestra
   // registra lascoltatore degli eventi della finestra
   addWindowListener(new AscoltatoreEventiFinestra());
   setSize(250,200); // dimensionamento finestra
   setResizable(false);
   setVisible(true);
   setLocation(200,150); // impostazione posizione iniziale
   setLayout(new BorderLayout()); // gestore aspetto finestra
   // creazione pulsanti
   for (int i=0; i<16; i++) {
    pulsanti[i] = new Button(tasti[i]);
   }
   pulsanti[16] = new Button("C");
   pulsanti[17] = new Button("CE");
   pulsanti[18] = new Button("=");
   // registrazione ascoltatore eventi dei pulsanti
   for (int i=0; i<19; i++) {
    pulsanti[i].addActionListener(this); 
   }
   Panel superiore = new Panel(); // pannello per display
   superiore.add(display);
   display.setEditable(false); // disabilitazione inserimento diretto
   Panel centrale = new Panel(); // pannello tastiera operativa
   centrale.setLayout(new GridLayout(4, 2, 4, 2));
   for (int i=0; i<16; i++) {
     centrale.add(pulsanti[i]);
   }
   Panel inferiore = new Panel(); // pannello pulsanti di controllo
   inferiore.setLayout(new GridLayout(1, 0, 1, 3));
   inferiore.add(pulsanti[16]);
   inferiore.add(pulsanti[17]);
   inferiore.add(pulsanti[18]);
   add("North", superiore);
   add("South", inferiore);
   add("Center", centrale);
   // oggetto che realizza la logica di business
   calcolatrice = new Calcolatrice();
   nuovoNumero = true;
   decimali = false;
  }
  // gestione eventi pulsanti
  public void actionPerformed(ActionEvent event){
    String string = event.getActionCommand(); // pulsante che ha generato levento
    // cambio segno
    if (string.equals("+/-")) {
      if (!nuovoNumero)
       calcolatrice.cambiaSegno();
      display.setText(Double.toString(calcolatrice.getOperando()));
    }
    // cifra numerica
    else if ( (string.equals("1")) || (string.equals("2")) ||
              (string.equals("3")) || (string.equals("4")) ||
              (string.equals("5")) || (string.equals("6")) ||
              (string.equals("7")) || (string.equals("8")) ||
              (string.equals("9")) || (string.equals("0"))) {
               if (display.getText().length() < NUMERO_MASSIMO_CIFRE) {
                 if (nuovoNumero) {
                    display.setText(string);
                    nuovoNumero = false;
                 }
                 else {
                    string = display.getText()+string;
                    display.setText(string);
                 }
                 calcolatrice.setOperando(Double.parseDouble(string));
               }
          }
          // operatore aritmetico
          else if ( (string.equals("+")) || (string.equals("-")) ||
                    (string.equals("*")) || (string.equals("/"))) {
                       calcolatrice.setOperazione(string.charAt(0));
                       nuovoNumero = true;
                       decimali = false;
               }
           // operatore uguale
           else if (string.equals("=")) {
                calcolatrice.uguale();
                display.setText(calcolatrice.risultato());
                nuovoNumero = true;
                decimali = false;
                calcolatrice.setOperazione('#');
           }
           // cancellazione ultimo operando inserito
           else if (string.equals("CE")) {
             calcolatrice.cancellaUltimoOperando();
             display.setText("0");
             decimali = false;
             nuovoNumero = true;
           }
           // cancellazione generale
           else if (string.equals("C")) {
              calcolatrice.cancella();
              display.setText("0");
              decimali = false;
              nuovoNumero = true;
           }
           // separatore decimale
           else if (string.equals(".")) {
             if ((!decimali) && (!nuovoNumero)) {
               decimali = true;
               string = display.getText()+".";
               display.setText(string);
             }
           }
  }
  public static void main(String args[]) {
    CalcolatriceGUI c = new CalcolatriceGUI();
  }
}